/* ThinkPad S230U finger events to xsmouse.sys */
rc=RxFuncAdd('SysGetMessage','RexxUtil','SysGetMessage')

/* set log file name */
ddNameR='bfinger.log'

/* report bfinger.cmd started */
info='<============<bfinger.cmd> started on <'||date()||'> at <'||time()'>============>'
rc=lineout(ddNameR,info) ; say info

/* set device driver names */
ddNameI='$FINGER$' ; ddNameO='XSMOUSE$'

/* verify that the direct control driver exists in config.sys */
if stream(ddNameI,'command','query exists') \= '\DEV\' || ddNameI
then do
  info='DEVICE=?:\OS2\BOOT\USBECD.SYS required in CONFIG.SYS'
  rc=lineout(ddNameR,info) ; say info
  /* wait */
  '@pause'
  exit
  end

/* acquire the direct control driver */
rc=stream(ddNameI,'command','open write')
if rc \= 'READY:'
then do
  info=rc||' Device driver '||ddNameI||' currently in use. Please try later.'
  rc=lineout(ddNameR,info) ; say info
  /* wait */
  '@pause'
  exit
  end

/* set configuration - configured */
rc=charout(ddNameI,x2c(00 09 01 00 00 00 00 00))
rc=stream(ddNameI,'description')
if rc \= 'READY:'
then do
  /* obtain and issue error message */
  parse value rc with sState ':' mNumber
  say SysGetMessage(mNumber,,ddNameI)
  say 'set configuration failed!'
  /* wait */
  '@pause'
  exit
  end

/* verify that the emulate mouse driver exists in config.sys */
if stream(ddNameO,'command','query exists') \= '\DEV\' || ddNameO
then do
  info='DEVICE=?:\OS2\BOOT\XSMOUSE.SYS required in CONFIG.SYS'
  rc=lineout(ddNameR,info) ; say info
  /* wait */
  '@pause'
  exit
  end

/* acquire the emulate mouse driver */
rc=stream(ddNameO,'command','open')
if rc \= 'READY:'
then do
  info=rc||' Device driver '||ddNameO||' currently in use. Please try later.'
  rc=lineout(ddNameR,info) ; say info
  /* wait */
  '@pause'
  exit
  end

/* set finger fixed extends */
xMinFinger=0 ; xMaxFinger=4095
yMinFinger=0 ; yMaxFinger=4095

/* initialize mouse events */
But1Down=x2c(0600) ; But2Down=x2c(1800)
MoveOnly=x2c(0100) ; NoButMov=x2c(0000)

/* set screen fixed extends */
xMaxScreen=1365 ; yMaxScreen=767

/* move cursor to center */
xMin=0 ; xMax=xMaxScreen ; xPos=trunc(xMax/2)
yMin=0 ; yMax=yMaxScreen ; yPos=trunc(yMax/2)
xMaxOut=reverse(d2c(xMaxScreen,2))
yMaxOut=reverse(d2c(yMaxScreen,2))
call xsMouse(MoveOnly)

/* use finger extends */
xMin=xMinFinger ; xMax=xMaxFinger
yMin=yMinFinger ; yMax=yMaxFinger

/* initialize */
oiBuffer = substr(x2c(EC 00 00 00 83 03 40 00),1,8+64,x2c(EE))

/* catch these here to release the mouse */
signal on error; signal on failure; signal on halt;
signal on novalue; signal on syntax;

do forever
  call GetPacket
  xPos=c2d(substr(oiBuffer,11,1))+256*c2d(substr(oiBuffer,12,1))
  yPos=c2d(substr(oiBuffer,15,1))+256*c2d(substr(oiBuffer,16,1))
  event1=substr(oiBuffer,10,1) ; event2=substr(oiBuffer,21,1)
  select
    when event2=x2c(14) then call xsMouse(But1Down)
    when event2=x2c(17) then call xsMouse(But2Down)
    when event1=x2c(0C) then call xsMouse(NoButMov)
    when event1=x2c(0D) then call xsMouse(But1Down)
    when event1=x2c(14) then call xsMouse(NoButMov)
    when event1=x2c(17) then call xsMouse(NoButMov)
    otherwise call Unknown
    end
  end

/* cleanup */
signal ProcessComplete
exit

GetPacket:
rc=charout(ddNameI,oiBuffer)
rc=stream(ddNameI,'description')
if rc \= 'READY:'
then do
  /* obtain error message */
  parse value rc with sState ':' mNumber
  if mNumber == 95 /* character i/o call interrupted */
  then rc=stream(ddNameI,'command','open') /* retry */
  else do
    /* issue error message */
    say SysGetMessage(mNumber,,ddNameI)
    say 'get finger packet failed!'
    /* wait */
    '@pause'
    exit
    end
  end
return

Unknown:
/* show packet */
packet=c2x(substr(oiBuffer,9))
output='Packet='||packet||',Huh?'
rc=lineout(ddNameR,output)
say output
return

xsMouse:
parse arg EvntFlgs
/* calculate current screen position */
xCur=trunc(((xPos-xMin)*xMaxScreen)/(xMax-xMin))
yCur=trunc(((yPos-yMin)*yMaxScreen)/(yMax-yMin))

/* prepare event buffer */
xPosOut=reverse(d2c(xCur,2))
yPosOut=reverse(d2c(yCur,2))
oBuffer = EvntFlgs||yPosOut||xPosOut||yMaxOut||xMaxOut

/* write this event */
rc=charout(ddNameO,oBuffer)
rc=stream(ddNameO,'description')
/* check completion code */
if rc \= 'READY:'
then do
  say
  /* obtain and issue error message */
  parse value rc with sState ':' mNumber
  say SysGetMessage(mNumber,,ddNameO)
  /* wait */
  '@pause'
  exit
  end
return

/* report signal information */
error: failure: halt: novalue: syntax:
parse source system invokation filename
info=condition('c')||' condition raised at line '||sigl||' of'||'0D0A'x
info=info||filename||'0D0A'x||'|'||sourceline(sigl)
rc=lineout(ddNameR,info) ; say info

ProcessComplete:
/* release the emulate mouse driver */
rc=stream(ddNameO,'command','close')
/* release the direct control driver */
rc=stream(ddNameI,'command','close')

/* report bfinger.cmd stopped */
info='<============<bfinger.cmd> stopped on <'||date()||'> at <'||time()'>============>'
rc=lineout(ddNameR,info) ; say info
exit
